@@ -1,7 +1,13 @@
 import datetime
 import logging
-import click
 import json
+import concurrent.futures
+import time
+
+import click
+import github
+
+from concurrent.futures import ThreadPoolExecutor
 
 from github import Github
 from terminaltables import AsciiTable
@@ -33,24 +39,31 @@ logger = logging.getLogger(__name__)
     default="asc",
     type=click.Choice(["desc", "asc"])
 )
+@click.option(
+    "--parallel",
+    default=10,
+    type=click.IntRange(1, 100)
+)
 @click.pass_context
-def cli(ctx, token, user, password, ignore, include, output_format, order):
+def cli(ctx, token, user, password, ignore, include, output_format, order,
+        parallel):
     ctx.ensure_object(dict)
 
     ctx.obj["output_format"] = output_format
     ctx.obj["order"] = order
+    ctx.obj["parallel_workers"] = parallel
 
     if token:
-        github = Github(token)
+        g = Github(token)
     else:
-        github = Github(user, password)
+        g = Github(user, password)
 
-    repos = github.get_user().get_repos()
+    repos = get_repos(g)
 
     ignore_repo_names = {x.strip() for x in ignore.split(",") if x.strip()}
     include_repo_names = {x.strip() for x in include.split(",") if x.strip()}
 
-    ctx.obj["github"] = github
+    ctx.obj["github"] = g
     ctx.obj["repos"] = list(
       x for x in filter_traffic_visible(repos)
       if x.name not in ignore_repo_names
@@ -76,6 +89,7 @@ def summary(ctx, metrics, days):
     repos = ctx.obj.get("repos")
     output_format = ctx.obj.get("output_format")
     order = ctx.obj.get("order")
+    parallel_workers = ctx.obj.get("parallel_workers")
 
     metrics = set(metrics)
 
@@ -91,29 +105,19 @@ def summary(ctx, metrics, days):
     fake_traffic = list(get_repos_zero_traffic(repos, dates))
 
     if show_views:
-        p = progressbar(
-            repos,
-            show_eta=False,
-            label="Fetching views stats",
-            item_show_func=lambda r: r and r.name
-        )
-        with p:
-            repos_views = list(get_repos_views_traffic(p, dates))
+        repos_views = list(get_repos_views_traffic(repos, dates, parallel_workers))
+
     else:
         repos_views = fake_traffic
 
     if show_clones:
-        p = progressbar(
-            repos,
-            show_eta=False,
-            label="Fetching clones stats",
-            item_show_func=lambda r: r and r.name
-        )
-        with p:
-            repos_clones = list(get_repos_clones_traffic(p, dates))
+        repos_clones = list(get_repos_clones_traffic(repos, dates, parallel_workers))
     else:
         repos_clones = fake_traffic
 
+    repos_views = sorted(repos_views, key=lambda r: r["name"])
+    repos_clones = sorted(repos_clones, key=lambda r: r["name"])
+
     if output_format == "json":
         out = {
             "views":  repos_views if show_views else None,
@@ -138,26 +142,47 @@ def referrers(ctx):
     repos = ctx.obj.get("repos")
     output_format = ctx.obj.get("output_format")
     order = ctx.obj.get("order")
+    parallel_workers = ctx.obj.get("parallel_workers")
 
     prog = progressbar(
-        repos,
+        length=len(repos),
         show_eta=False,
         label="Fetching referrers stats",
         item_show_func=lambda r: r and r.name
     )
-    with prog:
-        referrers = [
-            {
-                "repo": repo.name,
-                "referrer": r.referrer,
-                "count": r.count,
-                "uniques": r.uniques
-            } for repo in prog for r in repo.get_top_referrers()
+    with prog, ThreadPoolExecutor(max_workers=parallel_workers) as executor:
+        def get(repo):
+            try:
+                resp = repo.get_top_referrers()
+            except github.RateLimitExceededException as e:
+                retry_after = int(e.headers.get("retry-after", 0))
+                if retry_after:
+                    time.sleep(retry_after)
+                    resp = repo.get_top_referrers()
+                else:
+                    raise e
+
+            return resp, repo
+
+        futures = (executor.submit(get, repo) for repo in repos)
+
+        referrers = []
+        for f in concurrent.futures.as_completed(futures):
+            top_refs, repo = f.result()
+            prog.update(1, repo)
+            for r in top_refs:
+                referrers.append(
+                    {
+                        "repo": repo.name,
+                        "referrer": r.referrer,
+                        "count": r.count,
+                        "uniques": r.uniques
+                    }
+                )
 
-        ]
     referrers = sorted(
       referrers,
-      key=lambda x: (x["uniques"], x["count"]),
+      key=lambda x: (x["uniques"], x["count"], x["repo"]),
       reverse= True if order == "desc" else False
     )
 
@@ -186,27 +211,48 @@ def paths(ctx):
     repos = ctx.obj.get("repos")
     output_format = ctx.obj.get("output_format")
     order = ctx.obj.get("order")
+    parallel_workers = ctx.obj.get("parallel_workers")
 
     prog = progressbar(
-        repos,
+        length=len(repos),
         show_eta=False,
         label="Fetching paths stats",
         item_show_func=lambda r: r and r.name
     )
-    with prog:
-        paths = [
-            {
-                "repo": repo.name,
-                "path": p.path,
-                "title": p.title,
-                "count": p.count,
-                "uniques": p.uniques
-            } for repo in prog for p in repo.get_top_paths()
+    with prog, ThreadPoolExecutor(max_workers=parallel_workers) as executor:
+        def get(repo):
+            try:
+                resp = repo.get_top_paths()
+            except github.RateLimitExceededException as e:
+                retry_after = int(e.headers.get("retry-after", 0))
+                if retry_after:
+                    time.sleep(retry_after)
+                    resp = repo.get_top_paths()
+                else:
+                    raise e
+
+            return resp, repo
+
+        futures = (executor.submit(get, repo) for repo in repos)
+
+        paths = []
+        for f in concurrent.futures.as_completed(futures):
+            top_paths, repo  = f.result()
+            prog.update(1, repo)
+            for p in top_paths:
+                paths.append(
+                    {
+                        "repo": repo.name,
+                        "path": p.path,
+                        "title": p.title,
+                        "count": p.count,
+                        "uniques": p.uniques
+                    }
+                )
 
-        ]
     paths = sorted(
       paths,
-      key=lambda x: (x["uniques"], x["count"]),
+      key=lambda x: (x["uniques"], x["count"], x["repo"]),
       reverse= True if order == "desc" else False
     )
 
@@ -235,6 +281,18 @@ def progressbar(*args, **kwargs):
     return click.progressbar(*args, file=stderr_fobj, **kwargs)
 
 
+def get_repos(g):
+  try:
+      repos = list(g.get_user().get_repos())
+  except github.RateLimitExceededException as e:
+      retry_after = int(e.headers.get("retry-after", 0))
+      if retry_after:
+          time.sleep(retry_after)
+          repos = list(g.get_user().get_repos())
+      else:
+          raise e
+  return repos
+
 def filter_traffic_visible(repos):
     for repo in repos:
         if repo.permissions.push or repo.permissions.admin:
@@ -267,28 +325,74 @@ def traffic_on_dates(traffic, dates):
             }
 
 
-def get_repos_views_traffic(repos, breakdown_dates):
-    for repo in repos:
-        traffic = repo.get_views_traffic()
-        breakdown = list(traffic_on_dates(traffic["views"], breakdown_dates))
-        yield {
-            "name": repo.name,
-            "uniques": traffic["uniques"],
-            "count": traffic["count"],
-            "breakdown": breakdown
-        }
+def get_repos_views_traffic(repos, breakdown_dates, parallel_workers):
+    prog = progressbar(
+        length=len(repos),
+        show_eta=False,
+        label="Fetching views stats",
+        item_show_func=lambda r: r and r.name
+    )
+    with prog, ThreadPoolExecutor(max_workers=parallel_workers) as executor:
+        def get(repo):
+            try:
+                resp = repo.get_views_traffic()
+            except github.RateLimitExceededException as e:
+                retry_after = int(e.headers.get("retry-after", 0))
+                if retry_after:
+                    time.sleep(retry_after)
+                    resp = repo.get_views_traffic()
+                else:
+                    raise e
+
+            return resp, repo
+
+        futures = (executor.submit(get, repo) for repo in repos)
+
+        for f in concurrent.futures.as_completed(futures):
+            traffic, repo = f.result()
+            prog.update(1, repo)
+            breakdown = list(traffic_on_dates(traffic["views"], breakdown_dates))
+            yield {
+                "name": repo.name,
+                "uniques": traffic["uniques"],
+                "count": traffic["count"],
+                "breakdown": breakdown
+            }
 
 
-def get_repos_clones_traffic(repos, breakdown_dates):
-    for repo in repos:
-        traffic = repo.get_clones_traffic()
-        breakdown = list(traffic_on_dates(traffic["clones"], breakdown_dates))
-        yield {
-            "name": repo.name,
-            "uniques": traffic["uniques"],
-            "count": traffic["count"],
-            "breakdown": breakdown
-        }
+def get_repos_clones_traffic(repos, breakdown_dates, parallel_workers):
+    prog = progressbar(
+        length=len(repos),
+        show_eta=False,
+        label="Fetching clones stats",
+        item_show_func=lambda r: r and r.name
+    )
+    with prog, ThreadPoolExecutor(max_workers=parallel_workers) as executor:
+        def get(repo):
+            try:
+                resp = repo.get_clones_traffic()
+            except github.RateLimitExceededException as e:
+                retry_after = int(e.headers.get("retry-after", 0))
+                if retry_after:
+                    time.sleep(retry_after)
+                    resp = repo.get_clones_traffic()
+                else:
+                    raise e
+
+            return resp, repo
+
+        futures = (executor.submit(get, repo) for repo in repos)
+
+        for f in concurrent.futures.as_completed(futures):
+            traffic, repo = f.result()
+            prog.update(1, repo)
+            breakdown = list(traffic_on_dates(traffic["clones"], breakdown_dates))
+            yield {
+                "name": repo.name,
+                "uniques": traffic["uniques"],
+                "count": traffic["count"],
+                "breakdown": breakdown
+            }
 
 
 def get_repos_zero_traffic(repos, breakdown_dates):
